/****************************************************************************
** Meta object code from reading C++ file 'board.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/cutemaze-1.2.1/src/board.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'board.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_Board_t {
    QByteArrayData data[25];
    char stringdata0[238];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Board_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Board_t qt_meta_stringdata_Board = {
    {
QT_MOC_LITERAL(0, 0, 5), // "Board"
QT_MOC_LITERAL(1, 6, 13), // "hintAvailable"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 9), // "available"
QT_MOC_LITERAL(4, 31, 12), // "pauseChecked"
QT_MOC_LITERAL(5, 44, 7), // "checked"
QT_MOC_LITERAL(6, 52, 14), // "pauseAvailable"
QT_MOC_LITERAL(7, 67, 3), // "run"
QT_MOC_LITERAL(8, 71, 15), // "zoomInAvailable"
QT_MOC_LITERAL(9, 87, 16), // "zoomOutAvailable"
QT_MOC_LITERAL(10, 104, 8), // "finished"
QT_MOC_LITERAL(11, 113, 7), // "seconds"
QT_MOC_LITERAL(12, 121, 5), // "steps"
QT_MOC_LITERAL(13, 127, 9), // "algorithm"
QT_MOC_LITERAL(14, 137, 4), // "size"
QT_MOC_LITERAL(15, 142, 7), // "newGame"
QT_MOC_LITERAL(16, 150, 8), // "loadGame"
QT_MOC_LITERAL(17, 159, 8), // "saveGame"
QT_MOC_LITERAL(18, 168, 9), // "pauseGame"
QT_MOC_LITERAL(19, 178, 6), // "paused"
QT_MOC_LITERAL(20, 185, 4), // "hint"
QT_MOC_LITERAL(21, 190, 6), // "zoomIn"
QT_MOC_LITERAL(22, 197, 7), // "zoomOut"
QT_MOC_LITERAL(23, 205, 12), // "loadSettings"
QT_MOC_LITERAL(24, 218, 19) // "updateStatusMessage"

    },
    "Board\0hintAvailable\0\0available\0"
    "pauseChecked\0checked\0pauseAvailable\0"
    "run\0zoomInAvailable\0zoomOutAvailable\0"
    "finished\0seconds\0steps\0algorithm\0size\0"
    "newGame\0loadGame\0saveGame\0pauseGame\0"
    "paused\0hint\0zoomIn\0zoomOut\0loadSettings\0"
    "updateStatusMessage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Board[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    1,   92,    2, 0x06 /* Public */,
       6,    1,   95,    2, 0x06 /* Public */,
       8,    1,   98,    2, 0x06 /* Public */,
       9,    1,  101,    2, 0x06 /* Public */,
      10,    4,  104,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,  113,    2, 0x0a /* Public */,
      16,    0,  114,    2, 0x0a /* Public */,
      17,    0,  115,    2, 0x0a /* Public */,
      18,    1,  116,    2, 0x0a /* Public */,
      20,    0,  119,    2, 0x0a /* Public */,
      21,    0,  120,    2, 0x0a /* Public */,
      22,    0,  121,    2, 0x0a /* Public */,
      23,    0,  122,    2, 0x0a /* Public */,
      24,    0,  123,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,   11,   12,   13,   14,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Board::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Board *_t = static_cast<Board *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->hintAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->pauseChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->pauseAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->zoomInAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->zoomOutAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->finished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 6: _t->newGame(); break;
        case 7: _t->loadGame(); break;
        case 8: _t->saveGame(); break;
        case 9: _t->pauseGame((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->hint(); break;
        case 11: _t->zoomIn(); break;
        case 12: _t->zoomOut(); break;
        case 13: _t->loadSettings(); break;
        case 14: _t->updateStatusMessage(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::hintAvailable)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::pauseChecked)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::pauseAvailable)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::zoomInAvailable)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::zoomOutAvailable)) {
                *result = 4;
                return;
            }
        }
        {
            typedef void (Board::*_t)(int , int , int , int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::finished)) {
                *result = 5;
                return;
            }
        }
    }
}

const QMetaObject Board::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_Board.data,
      qt_meta_data_Board,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *Board::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Board::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_Board.stringdata0))
        return static_cast<void*>(const_cast< Board*>(this));
    return QWidget::qt_metacast(_clname);
}

int Board::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Board::hintAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Board::pauseChecked(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Board::pauseAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Board::zoomInAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Board::zoomOutAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Board::finished(int _t1, int _t2, int _t3, int _t4)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_END_MOC_NAMESPACE
